import java.io.*;
import java.util.regex.*;
/**
 * Export the schema of a database to a file
 */

public class ExportDatatoOWL {

	private Database db;
	private Config owlTemplate;
	private String saveFile;
	private String schemaFile;

	/** 
	 * Export the Data to an OWL file
	 * @param db the database
	 * @param saveFile the file where to save the OWL document
	 * @param schemaFile
	 * @throws Exception
	 */
	public ExportDatatoOWL(Database db, String saveFile, String schemaFile) throws Exception {
		this.db = db;
		this.saveFile = saveFile;
		this.schemaFile = schemaFile;
		db.getConnection();
		ExportData();
	}
	/** ***************************************************************************************************** */
	public void ExportData() throws Exception {
		long timeStart = System.currentTimeMillis();
		System.out.println("[+] Exporting Data ");
		owlTemplate = new Config("DataOWL.properties");
		FileWriter schemaFile = new FileWriter(saveFile);
		schemaFile.write(Header());
		String tnames[] = db.getTableNames();
		for (int i = 0; i < tnames.length; i++)
			schemaFile.write(TableData(tnames[i]));
		schemaFile.write(Footer());
		schemaFile.close();
		float duration = (System.currentTimeMillis() - timeStart) / 1000;
		System.out.println("[+] Exporting Data :" + duration + "s");
	}
	/**
	 * @return the generic header of the OWL Document
	 */
	public String Header() {
		String outputStr = Replace(owlTemplate.getProperty("Header"), "SCHEMA_FILE_NAME", this.schemaFile);
		return outputStr;
	}
	/**
	 * @return the generic footer of the OWL Document
	 */
	public String Footer() {
		return (owlTemplate.getProperty("Footer"));
	}
	/** ***************************************************************************************************** */
	public String TableData(String tableName) {
		String outputStr = "";
		String value = "";

		String cnames[] = db.getColumnNames(tableName);

		// 2004_03_23 : TODO : Oulala ! for sure not the best code, but ....
		String tbvalues[] = db.getColumnValues(tableName, cnames[0]);

		for (int j = 0; j < tbvalues.length; j++) {
			outputStr += Replace(owlTemplate.getProperty("DataH"), "TABLE_NAME", tableName);
			//System.out.println("-j-"+j);
			for (int i = 0; i < cnames.length; i++) {
				tbvalues = db.getColumnValues(tableName, cnames[i]);
				value = tbvalues[j];
				outputStr += Replace(Replace(owlTemplate.getProperty("DataR"), "COLUMN_NAME", tableName + "." + cnames[i]), "DATA_VALUE", value);				
			}
			outputStr += Replace(owlTemplate.getProperty("DataF"), "TABLE_NAME", tableName);
		}
		return (outputStr);
	}

	/**
	 * A basic search and replace string function
	 * 
	 * @param inputStr :
	 *                  input string
	 * @param inputPattern :
	 *                  input pattern
	 * @param replaceStr :
	 *                  replace string
	 * @return
	 */
	private String Replace(String inputStr, String inputPattern, String replaceStr) {
		String outputStr = "";
		try {
			Pattern pattern = Pattern.compile(inputPattern);
			Matcher matcher = pattern.matcher(inputStr);
			outputStr = matcher.replaceAll(replaceStr);
		} catch (Exception e) {
			System.out.println("Error Replace ");
		}
		return outputStr;
	}
}
